//-----------------------------------------------------------------------------
// Window
//
// This is a simple application that creates a window and handle its messages
// until the user decides to close it. It does nothing else and is only 
// intended as an introduction to the Win32 API.
//
// To compile this source code you need to link with user32.lib and gdi32.lib
//
// Copyright (c) 2000, Andreas Jnsson 
//-----------------------------------------------------------------------------

// Don't include rarely used code to speed up compilation
#define WIN32_LEAN_AND_MEAN 

#include <windows.h>



//-----------------------------------------------------------------------------
// Prototypes
//-----------------------------------------------------------------------------
HRESULT Create(HINSTANCE hInst);
int     Run();
LRESULT CALLBACK WndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);




//-----------------------------------------------------------------------------
// WinMain()
//
// This is the entry point for any Win32 application.
//-----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmdLine, int nShowCmd)
{
	// Create the window
	HRESULT hr;
	if( FAILED(hr = Create(hInst)) )
		return hr;

	// Run the application
	return Run();
}



//-----------------------------------------------------------------------------
// Create()
//
// In this function we create the window that will be shown to the user
//-----------------------------------------------------------------------------
HRESULT Create(HINSTANCE hInst)
{
    // Register a window class
    WNDCLASS WndClass;
	WndClass.style         = CS_HREDRAW |      // Redraw if moved or resized
		                     CS_VREDRAW | 
							 CS_OWNDC;         // Unique device context 
	WndClass.lpfnWndProc   = (WNDPROC)WndProc; // Our message handler
	WndClass.cbClsExtra    = 0;                // No extra memory needed
	WndClass.cbWndExtra    = 0;                // No extra memory needed
	WndClass.hInstance     = hInst;            // App instance
	WndClass.hIcon         = LoadIcon(NULL, IDI_WINLOGO); // Standard icon
	WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW); // Standard cursor
	WndClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);  
	WndClass.lpszMenuName  = NULL;             // No menu
	WndClass.lpszClassName = "Application Window"; 

    if( !RegisterClass( &WndClass ) )
		return E_FAIL;

    // Create the render window
    HWND hWnd = CreateWindow( "Application Window", // The window class 
		                      "Test Window",        // The window title
							  WS_OVERLAPPEDWINDOW,  // Normal window style
							  CW_USEDEFAULT,        // Default position
							  CW_USEDEFAULT,
			  		   	      640,					// The size of the window
							  480,
							  NULL,                 // No parent window
							  NULL,                 // No menu
							  hInst,                // The app instance
							  NULL );               // No extra info
	if( hWnd == NULL )
		return E_FAIL;

	// Show the window
	ShowWindow( hWnd, SW_SHOW );
    UpdateWindow( hWnd );

	return S_OK;
}




//-----------------------------------------------------------------------------
// Run()
// 
// Here we receive the messages that are sent to us from the OS
//-----------------------------------------------------------------------------
int Run()
{
	// Check messages until we receive a WM_QUIT message
    MSG  Msg;
	while( GetMessage( &Msg, NULL, 0U, 0U ) )
    {
        // Dispatch the message to the window procedure
        DispatchMessage( &Msg );
    }

    return Msg.wParam;
}




//-----------------------------------------------------------------------------
// WndProc()
//
// In the window procedure the messages are processed
//-----------------------------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
    switch( uMsg )
	{
	case WM_DESTROY:
		// Tell the message loop to quit
		PostQuitMessage(0);
		return 0;
	}

	// Let the default window procedure handle 
	// any message that we don't care about
    return DefWindowProc( hWnd, uMsg, wParam, lParam );
}
